package recorder

import (
	"context"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/trace"

	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

func (w *WeatherRecorder) Set(ctx context.Context, u *weather.WeatherUpdate) error {
	ctx, span := w.tracer.Start(ctx, "recordWeatherUpdate")
	span.SetAttributes(
		attribute.Int("countWeatherUpdates", w.Count(ctx)),
		attribute.Int("keepUpdates", w.keep),
	)
	defer span.End()

	return w.set(span, u)
}

func (w *WeatherRecorder) set(span trace.Span, u *weather.WeatherUpdate) error {
	w.Lock()
	defer w.Unlock()

	if len(w.updates) > w.keep {
		w.updates = w.updates[1:]
		span.AddEvent("trimmed recorded updates by 1")
	}

	w.updates = append(w.updates, u)
	span.AddEvent("recorded weather update")
	return nil
}
